"use strict";

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

(function () {
  /*
   * Federated Wiki : Bytebeat Plugin
   *
   * Licensed under the MIT license.
   * https://github.com/fedwiki/wiki-plugin-bytebeat/blob/master/LICENSE.txt
   */
  var FMTSubChunk, RIFFChunk, audioCheck, b, _c, chunkSize, colorCode, dataSubChunk, frequency, generateSound, makeSampleFunction, makeURL, sampleArrayToData, split32bitValueToBytes;

  window.plugins.bytebeat = {
    emit: function emit(div, item) {
      div.append("<p>".concat(colorCode(item.text), " <a href='#' class='play'>&#9654;</a></div></p>"));
      return audioCheck();
    },
    bind: function bind(div, item) {
      var _this = this;

      div.find('a').click(function () {
        return _this.play(div, item);
      });
      return div.dblclick(function () {
        _this.stop(div);

        return wiki.textEditor(div, item);
      });
    },
    play: function play(div, item) {
      this.stop(div);
      return $("<audio>").attr({
        autoplay: true,
        src: makeURL(item.text),
        controls: "controls"
      }).appendTo(div);
    },
    stop: function stop(div) {
      return this.audio(div).remove();
    },
    audio: function audio(div) {
      return div.find("audio");
    }
  };

  colorCode = function colorCode(text) {
    return text.replace(/\bt((<<|>>)\d+)?\b/g, function (m) {
      return "<span class='symbol'>".concat(m, "</span>");
    }).replace(/\n/g, '<br>');
  };

  audioCheck = function audioCheck() {
    var elm;
    elm = document.createElement("audio");

    if (!_typeof(elm.play)) {
      throw "You don't seem to have a browser that supports audio.";
    }
  }; // Adapted from ...
  // Code in ~2 hours by Bemmu, idea and sound code snippet from Viznut.


  makeSampleFunction = function makeSampleFunction(text) {
    var js;
    js = text.replace(/sin/g, "Math.sin").replace(/cos/g, "Math.cos").replace(/tan/g, "Math.tan").replace(/floor/g, "Math.floor").replace(/ceil/g, "Math.ceil");
    eval("var f = function (t) {return ".concat(js, "}"));
    return f;
  };

  frequency = 8000;

  generateSound = function generateSound(f) {
    var results, samples, seconds, t;
    seconds = 32.767;
    samples = frequency * seconds;
    t = 0;
    results = [];

    while (t < samples) {
      results.push((f(t++) & 0xff) * 256);
    }

    return results;
  };

  b = function b(values) {
    var num, strings;

    strings = function () {
      var i, len, results;
      results = [];

      for (i = 0, len = values.length; i < len; i++) {
        num = values[i];
        results.push((num > 15 ? "%" : "%0") + num.toString(16));
      }

      return results;
    }();

    return strings.join('').toUpperCase();
  };

  _c = function c(str) {
    var code, i, len, results;

    if (str.length === 1) {
      return str.charCodeAt(0);
    } else {
      results = [];

      for (i = 0, len = str.length; i < len; i++) {
        code = str[i];
        results.push(_c(code));
      }

      return results;
    }
  };

  split32bitValueToBytes = function split32bitValueToBytes(l) {
    return [l & 0xff, (l & 0xff00) >> 8, (l & 0xff0000) >> 16, (l & 0xff000000) >> 24];
  };

  FMTSubChunk = function FMTSubChunk(channels, bitsPerSample, frequency) {
    var blockAlign, byteRate;
    byteRate = frequency * channels * bitsPerSample / 8;
    blockAlign = channels * bitsPerSample / 8;
    return [].concat(_c("fmt "), split32bitValueToBytes(16), [1, 0], [channels, 0], split32bitValueToBytes(frequency), split32bitValueToBytes(byteRate), [blockAlign, 0], [bitsPerSample, 0]);
  };

  sampleArrayToData = function sampleArrayToData(sampleArray, bitsPerSample) {
    var data, i, len, samp;

    if (bitsPerSample === 8) {
      return sampleArray;
    }

    if (bitsPerSample !== 16) {
      throw "Only 8 or 16 bit supported.";
    }

    data = [];

    for (i = 0, len = sampleArray.length; i < len; i++) {
      samp = sampleArray[i];
      data.push(0xff & samp);
      data.push((0xff00 & samp) >> 8);
    }

    return data;
  };

  dataSubChunk = function dataSubChunk(channels, bitsPerSample, sampleArray) {
    return [].concat(_c("data"), split32bitValueToBytes(sampleArray.length * channels * bitsPerSample / 8), sampleArrayToData(sampleArray, bitsPerSample));
  };

  chunkSize = function chunkSize(fmt, data) {
    return split32bitValueToBytes(4 + (8 + fmt.length) + (8 + data.length));
  };

  RIFFChunk = function RIFFChunk(channels, bitsPerSample, frequency, sampleArray) {
    var data, fmt, header;
    fmt = FMTSubChunk(channels, bitsPerSample, frequency);
    data = dataSubChunk(channels, bitsPerSample, sampleArray);
    header = [].concat(_c("RIFF"), chunkSize(fmt, data), _c("WAVE"));
    return [].concat(header, fmt, data);
  };

  makeURL = function makeURL(text) {
    var bitsPerSample, channels, samples;
    bitsPerSample = 16;
    samples = generateSound(makeSampleFunction(text));
    channels = 1;
    return "data:audio/x-wav," + b(RIFFChunk(channels, bitsPerSample, frequency, samples));
  };
}).call(void 0);
//# sourceMappingURL=bytebeat.js.map
